import uuid
import logging
from copy import copy
from datetime import datetime as py_datetime
from django.core.cache import caches
import datetime as base_datetime
from dirtyfields import DirtyFieldsMixin
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models import F
from simple_history.models import HistoricalRecords
from core.utils import (
    CachedManager,
    CachedModelMixin
)
#from core.datetimes.ad_datetime import datetime as py_datetime

from ..fields import DateTimeField
from .user import User

logger = logging.getLogger(__name__)

cache = caches["default"]

class HistoryModelManager(CachedManager):
    """
        Custom manager that allows querying HistoryModel by uuid
        and includes caching logic for better performance.
    """

    def get_queryset(self):
        return super().get_queryset().annotate(uuid=F('id'))
    

    def filter(self, *args, **kwargs):
        # Check if 'uuid' is in kwargs, and if so, rename it to 'id'
        if 'uuid' in kwargs:
            kwargs['id'] = kwargs.pop('uuid')
        # Call the parent class's filter method with the modified kwargs
        return super().filter(*args, **kwargs)
    
    def get(self, *args, **kwargs):
        # Check if 'uuid' is in kwargs, and if so, rename it to 'id'
        if 'uuid' in kwargs:
            kwargs['id'] = kwargs.pop('uuid')
        # Call the parent class's filter method with the modified kwargs
        return super().get(*args, **kwargs) 
    
    
class HistoryModel(DirtyFieldsMixin,CachedModelMixin, models.Model):
    id = models.UUIDField(primary_key=True, db_column="UUID", default=None, editable=False)
    objects = HistoryModelManager()
    is_deleted = models.BooleanField(db_column="isDeleted", default=False)
    json_ext = models.JSONField(db_column="Json_ext", blank=True, null=True)
    date_created = DateTimeField(db_column="DateCreated", null=True, default=py_datetime.now)
    date_updated = DateTimeField(db_column="DateUpdated", null=True, default=py_datetime.now)
    user_created = models.ForeignKey(User, db_column="UserCreatedUUID", related_name='%(class)s_user_created',
                                     on_delete=models.deletion.DO_NOTHING, null=False)
    user_updated = models.ForeignKey(User, db_column="UserUpdatedUUID", related_name='%(class)s_user_updated',
                                     on_delete=models.deletion.DO_NOTHING, null=False)
    version = models.IntegerField(default=1)
    history = HistoricalRecords(
        inherit=True,
    )


    @property
    def uuid(self):
        return self.id

    @uuid.setter
    def uuid(self, v):
        self.id = v

    def set_pk(self):
        self.pk = uuid.uuid4()

    def save_history(self):
        pass
    
    def update(self, *args, user=None, username=None, save=True, **kwargs):
        """
        Overrides the default update to update the cache after saving the instance.
        """
        obj_data = kwargs.pop('data', {})
        if not obj_data:
            obj_data = kwargs
            kwargs = {}
        [setattr(self, key, obj_data[key]) for key in obj_data]
        if save:
            self.save(*args, user=user, username=user, **kwargs)
        return self
        
    def save(self, *args, user=None, username=None, **kwargs):
        # get the user data so as to assign later his uuid id in fields user_updated etc
        if not user:
            if username:
                user = User.objects.get(username=username)
            else:
                raise ValidationError('Save error! Provide user or the username of the current user in `username` argument')
        now = py_datetime.now()
        # check if object has been newly created
        if self.id is None:
            # save the new object
            self.set_pk()
            self.user_created = user
            self.user_updated = user
            self.date_created = now
            self.date_updated = now
            instance = super(HistoryModel, self).save(*args, **kwargs)
            self.update_cache()
            return instance
        if self.is_dirty(check_relationship=True):
            if not self.user_created:
                past = self.objects.filter(pk=self.id).first()
                if not past:
                    self.user_created = user
                    self.user_updated = user
                    self.date_created = now
                    self.date_updated = now 
                # TODO this could erase a instance, version check might be too light
                elif not self.version == past.version:
                    raise ValidationError('Record has not be updated - the version don\'t match with existing record')
            self.date_updated = now
            self.user_updated = user
            self.version = self.version + 1
            # check if we have business model
            if hasattr(self, "replacement_uuid"):
                if self.replacement_uuid is not None and 'replacement_uuid' not in self.get_dirty_fields():
                    raise ValidationError('Update error! You cannot update replaced entity')
            instance = super(HistoryModel, self).save(*args, **kwargs)  
            self.update_cache()
            return instance  
        else:
            raise ValidationError('Record has not be updated - there are no changes in fields')

    def delete_history(self):
        pass

    def delete(self, *args, user=None, username=None, **kwargs):
        if not user:
            if username:
                user = User.objects.get(username=username)
            else:
                raise ValidationError('Save error! Provide user or the username of the current user in `username` argument')
        if not self.is_dirty(check_relationship=True) and not self.is_deleted:

            now = py_datetime.now()
            self.date_updated = now
            self.user_updated = user
            self.version = self.version + 1
            self.is_deleted = True
            # check if we have business model
            if hasattr(self, "replacement_uuid"):
                # When a replacement entity is deleted, the link should be removed
                # from replaced entity so a new replacement could be generated
                replaced_entity = self.__class__.objects.filter(replacement_uuid=self.id).first()
                if replaced_entity:
                    replaced_entity.replacement_uuid = None
                    replaced_entity.save(username="admin")
            instance = super(HistoryModel, self).save(*args, **kwargs)
            self.delete_cache()
            return instance  
        else:
            raise ValidationError(
                'Record has not be deactivating, the object is different and must be updated before deactivating')
        




    def copy(self, exclude_fields=['id', 'uuid']):
        """
        Creates a copy of a Django model instance, excluding specified fields (default: 'id' and 'uuid').
        Args:
            exclude_fields: List of field names to exclude from copying (default: ['id', 'uuid'])
        Returns:
            A new unsaved instance with copied attributes
        """
        model_class = self.__class__
        new_instance = model_class()
        fields = self._meta.get_fields()
        for field in fields:
            if field.name not in exclude_fields and hasattr(self, field.name):
                if field.is_relation:
                    if field.many_to_one or field.one_to_one:
                        setattr(new_instance, field.name, getattr(self, field.name))
                    elif field.one_to_many or field.many_to_many:
                        continue
                else:
                    setattr(new_instance, field.name, getattr(self, field.name))

        return new_instance

    @classmethod
    def filter_queryset(cls, queryset=None):
        if queryset is None:
            queryset = cls.objects.all()
        queryset = queryset.filter()
        return queryset

    class Meta:
        abstract = True


class HistoryBusinessModel(HistoryModel):
    date_valid_from = DateTimeField(db_column="DateValidFrom", default=py_datetime.now)
    date_valid_to = DateTimeField(db_column="DateValidTo", blank=True, null=True)
    replacement_uuid = models.UUIDField(db_column="ReplacementUUID", blank=True, null=True)

    def replace_object(self, data, **kwargs):
        # check if object was created and saved in database (having date_created field)
        if self.id is None:
            return None
        user = User.objects.get(**kwargs)
        # 1 step - create new entity
        new_entity = self._create_new_entity(user=user, data=data)
        # 2 step - update the fields for the entity to be replaced
        self._update_replaced_entity(user=user, uuid_from_new_entity=new_entity.id,
                                     date_valid_from_new_entity=new_entity.date_valid_from)

    def _create_new_entity(self, user, data):
        """1 step - create new entity"""
        now = py_datetime.now()
        new_entity = copy(self)
        new_entity.id = None
        new_entity.version = 1
        new_entity.date_valid_from = now
        new_entity.date_valid_to = None
        new_entity.replacement_uuid = None
        # replace the fiedls if there are any to update in new entity
        if "uuid" in data:
            data.pop('uuid')
        if len(data) > 0:
            [setattr(new_entity, key, data[key]) for key in data]
        if self.date_valid_from is None:
            raise ValidationError('Field date_valid_from should not be empty')
        new_entity.save(username=user.username)
        return new_entity

    def _update_replaced_entity(self, user, uuid_from_new_entity, date_valid_from_new_entity):
        """2 step - update the fields for the entity to be replaced"""
        # convert to datetime if the date_valid_from from new entity is date
        if not isinstance(date_valid_from_new_entity, base_datetime.datetime):
            date_valid_from_new_entity = base_datetime.combine(
                date_valid_from_new_entity,
                base_datetime.min.time()
            )
        if not self.is_dirty(check_relationship=True):
            if self.date_valid_to is not None:
                if date_valid_from_new_entity < self.date_valid_to:
                    self.date_valid_to = date_valid_from_new_entity
            else:
                self.date_valid_to = date_valid_from_new_entity
            self.replacement_uuid = uuid_from_new_entity
            self.save(username=user.username)
            self.delete_cache()
            return self
        else:
            raise ValidationError("Object is changed - it must be updated before being replaced")

    class Meta:
        abstract = True




import importlib
import logging

from django.db import connection
from django.test.client import RequestFactory
from django.apps import apps
import datetime
import core
from core.models import InteractiveUser, Officer, UserRole
from core.services import (
    create_or_update_interactive_user,
    create_or_update_core_user,
    create_or_update_officer,
    create_or_update_claim_admin,
    reset_user_password,
    set_user_password,
)
from django.test import TestCase
from location.models import OfficerVillage

logger = logging.getLogger(__file__)
postgresql = "postgresql"
PASSWORD = "FoBoar72!"


class UserServicesTest(TestCase):
    claim_admin_class = None

    def setUp(self):
        super(UserServicesTest, self).setUp()
        # This shouldn't be necessary but cleanup from date tests tend not to cleanup properly
        core.calendar = importlib.import_module(".calendars.ad_calendar", "core")
        core.datetime = importlib.import_module(".datetimes.ad_datetime", "core")
        self.claim_admin_class = apps.get_model("core", "ClaimAdmin")
        self.factory = RequestFactory()

    def test_iuser_min(self):
        roles = [11]
        username = "tstsvciu1"
        i_user, created = create_or_update_interactive_user(
            user_id=None,
            data=dict(
                username=username,
                last_name="Last Name CIU1",
                other_names="Other 1 2 3",
                roles=roles,
                language="en",
            ),
            audit_user_id=999,
            connected=False,
        )
        self.assertTrue(created)
        self.assertIsNotNone(i_user)
        self.assertEquals(i_user.username, username)
        self.assertEquals(i_user.last_name, "Last Name CIU1")
        self.assertEquals(i_user.other_names, "Other 1 2 3")
        self.assertEquals(i_user.user_roles.count(), 1)
        self.assertEquals(i_user.user_roles.first().role_id, 11)
        self.assertEquals(i_user.language.code, "en")

        UserRole.objects.filter(user_id=i_user.id).delete()
        deleted_users = InteractiveUser.objects.filter(login_name=username).delete()
        logger.info(f"Deleted {deleted_users} users after test")

    def test_iuser_max(self):
        roles = [11, 12]
        username = "tstsvciu2"
        i_user, created = create_or_update_interactive_user(
            user_id=None,
            data=dict(
                username=username,
                last_name="Last Name CIU2",
                other_names="Other 1 2 3",
                roles=roles,
                language="fr",
                phone="+123456789",
                email=f"{username}@illuminati.int",
                health_facility_id=1,
                password=PASSWORD,
            ),
            audit_user_id=999,
            connected=False,
        )
        self.assertTrue(created)
        self.assertIsNotNone(i_user)
        self.assertEquals(i_user.username, username)
        self.assertEquals(i_user.last_name, "Last Name CIU2")
        self.assertEquals(i_user.other_names, "Other 1 2 3")
        self.assertEquals(i_user.user_roles.count(), 2)
        self.assertEquals(
            list(i_user.user_roles.values_list("role_id", flat=True)), roles
        )
        self.assertEquals(i_user.language.code, "fr")
        self.assertEquals(i_user.phone, "+123456789")
        self.assertEquals(i_user.email, f"{username}@illuminati.int")
        self.assertIsNotNone(i_user.password)
        self.assertNotEqual(i_user.password, PASSWORD)  # No clear text password
        self.assertTrue(i_user.check_password(PASSWORD))
        self.assertFalse(i_user.check_password("wrong_password"))

        UserRole.objects.filter(user_id=i_user.id).delete()
        deleted_users = InteractiveUser.objects.filter(login_name=username).delete()
        logger.info(f"Deleted {deleted_users} users after test")

    def test_iuser_update(self):
        roles = [11, 12]
        roles2 = [8, 11]
        username = "tstsvciu2"
        i_user, created = create_or_update_interactive_user(
            user_id=None,
            data=dict(
                username=username,
                last_name="Last Name CIU2",
                other_names="Other 1 2 3",
                roles=roles,
                language="fr",
                phone="+123456789",
                email=f"{username}@illuminati.int",
                health_facility_id=1,
                password=PASSWORD,
            ),
            audit_user_id=999,
            connected=False,
        )
        self.assertTrue(created)
        self.assertIsNotNone(i_user)
        self.assertEquals(i_user.username, username)
        self.assertEquals(i_user.last_name, "Last Name CIU2")
        self.assertEquals(i_user.other_names, "Other 1 2 3")
        self.assertEquals(i_user.user_roles.count(), 2)
        self.assertEquals(
            list(i_user.user_roles.values_list("role_id", flat=True)), roles
        )
        self.assertEquals(i_user.language.code, "fr")
        self.assertEquals(i_user.phone, "+123456789")
        self.assertEquals(i_user.email, f"{username}@illuminati.int")
        self.assertTrue(i_user.check_password(PASSWORD))

        # Core user necessary for the update
        core_user, core_user_created = create_or_update_core_user(
            None, username, i_user=i_user
        )

        i_user2, created2 = create_or_update_interactive_user(
            user_id=core_user.id,
            data=dict(
                username=username,
                last_name="Last updated",
                other_names="Other updated",
                roles=roles2,
                language="en",
                phone="updated phone",
                email=f"{username}@updated.int",
                health_facility_id=2,
                password=f"{PASSWORD}updated",
            ),
            audit_user_id=111,
            connected=False,
        )
        self.assertFalse(created2)
        self.assertIsNotNone(i_user2)
        self.assertEquals(i_user2.username, username)
        self.assertEquals(i_user2.last_name, "Last updated")
        self.assertEquals(i_user2.other_names, "Other updated")
        self.assertEquals(
            i_user2.user_roles.filter(validity_to__isnull=True).count(), 2
        )
        self.assertEquals(
            list(
                i_user2.user_roles.filter(validity_to__isnull=True)
                .order_by("role_id")
                .values_list("role_id", flat=True)
            ),
            roles2,
        )
        self.assertEquals(i_user2.language.code, "en")
        self.assertEquals(i_user2.phone, "updated phone")
        self.assertEquals(i_user2.email, f"{username}@updated.int")
        self.assertTrue(i_user2.check_password(f"{PASSWORD}updated"))

        UserRole.objects.filter(user_id=i_user.id).delete()
        core_user.delete()
        deleted_users = InteractiveUser.objects.filter(login_name=username).delete()
        logger.info(f"Deleted {deleted_users} users after test")

    def test_iuser_update_no_id(self):
        """
        This tests the update of a user without specifying a userId but with a username
        """
        roles = [11, 12]
        username = "tstsvciu3"
        i_user, created = create_or_update_interactive_user(
            user_id=None,
            data=dict(
                username=username,
                last_name="Last Name CIU2",
                other_names="Other 1 2 3",
                roles=roles,
                language="fr",
            ),
            audit_user_id=999,
            connected=False,
        )
        self.assertTrue(created)
        self.assertIsNotNone(i_user)
        self.assertEquals(i_user.username, username)

        i_user2, created2 = create_or_update_interactive_user(
            user_id=None,
            data=dict(
                username=username,
                last_name="Last updated",
                other_names="Other updated",
                language="en",
                roles=roles,
            ),
            audit_user_id=111,
            connected=False,
        )
        self.assertFalse(created2)
        self.assertIsNotNone(i_user2)
        self.assertEquals(i_user2.username, username)
        self.assertEquals(i_user2.last_name, "Last updated")
        self.assertEquals(i_user2.other_names, "Other updated")

        UserRole.objects.filter(user_id=i_user.id).delete()
        deleted_users = InteractiveUser.objects.filter(login_name=username).delete()
        logger.info(f"Deleted {deleted_users} users after test")

    def test_officer_min(self):
        username = "tstsvco1"
        officer, created = create_or_update_officer(
            user_id=None,
            data=dict(
                username=username, last_name="Last Name O1", other_names="Other 1 2 3", phone="+12345678"
            ),
            audit_user_id=999,
            connected=False,
        )
        self.assertTrue(created)
        self.assertIsNotNone(officer)
        self.assertEquals(officer.username, username)
        self.assertEquals(officer.last_name, "Last Name O1")
        self.assertEquals(officer.other_names, "Other 1 2 3")
        self.assertEquals(officer.phone, "+12345678")

        deleted_officers = Officer.objects.filter(code=username).delete()
        logger.info(f"Deleted {deleted_officers} officers after test")

    def test_officer_max(self):
        username = "tstsvco2"
        officer, created = create_or_update_officer(
            user_id=None,
            data=dict(
                username=username,
                last_name="Last Name O2",
                other_names="Other 1 2 3",
                dob="1999-05-05",
                phone="+12345678",
                email="imis@foo.be",
                location_id=1,
                village_ids=[22, 35, 50],
                substitution_officer_id=1,
                works_to="2025-01-01",
                phone_communication=True,
                address="Multi\nline\naddress",
            ),
            audit_user_id=999,
            connected=True,
        )
        self.assertTrue(created)
        self.assertIsNotNone(officer)
        self.assertEquals(officer.username, username)
        self.assertEquals(officer.last_name, "Last Name O2")
        self.assertEquals(officer.other_names, "Other 1 2 3")
        self.assertEquals(officer.audit_user_id, 999)
        self.assertTrue(officer.has_login)
        self.assertTrue(officer.phone_communication)
        self.assertEquals(officer.location_id, 1)
        self.assertEquals(officer.substitution_officer_id, 1)
        self.assertEquals(officer.address, "Multi\nline\naddress")
        self.assertEquals(str(officer.works_to.date()), "2025-01-01")
        self.assertEquals(
            list(
                OfficerVillage.objects.filter(validity_to__isnull=True, officer=officer)
                .order_by("location_id")
                .values_list("location_id", flat=True)
            ),
            [22, 35, 50],
        )
        self.assertEquals(officer.phone, "+12345678")
        self.assertEquals(officer.email, "imis@foo.be")

        deleted_officers = Officer.objects.filter(code=username).delete()
        logger.info(f"Deleted {deleted_officers} officers after test")

    def test_officer_update(self):
        username = "tstsvco2"
        officer, created = create_or_update_officer(
            user_id=None,
            data=dict(
                username=username,
                last_name="Last Name O2",
                other_names="Other 1 2 3",
                dob="1999-05-05",
                phone="+12345678",
                email="imis@foo.be",
                location_id=1,
                village_ids=[22, 35, 50],
                substitution_officer_id=1,
                works_to="2025-01-01",
                phone_communication=True,
                address="Multi\nline\naddress",
            ),
            audit_user_id=999,
            connected=True,
        )
        self.assertTrue(created)
        self.assertIsNotNone(officer)
        self.assertEquals(officer.username, username)
        self.assertEquals(officer.last_name, "Last Name O2")
        self.assertEquals(officer.other_names, "Other 1 2 3")
        self.assertEquals(officer.audit_user_id, 999)
        self.assertTrue(officer.has_login)
        self.assertTrue(officer.phone_communication)
        self.assertEquals(officer.location_id, 1)
        self.assertEquals(officer.substitution_officer_id, 1)
        self.assertEquals(officer.address, "Multi\nline\naddress")
        self.assertEquals(str(officer.works_to.date()), "2025-01-01")
        self.assertEquals(
            list(
                OfficerVillage.objects.filter(validity_to__isnull=True, officer=officer)
                .order_by("location_id")
                .values_list("location_id", flat=True)
            ),
            [22, 35, 50],
        )
        self.assertEquals(officer.phone, "+12345678")
        self.assertEquals(officer.email, "imis@foo.be")

        officer2, created = create_or_update_officer(
            user_id=None,
            data=dict(
                username=username,
                last_name="Last updated",
                other_names="Other updated",
                dob="1999-01-01",
                phone="+00000",
                email="imis@bar.be",
                location_id=17,
                village_ids=[22],
                substitution_officer_id=None,
                works_to=datetime.date(2025, 5, 5),
                phone_communication=False,
                address="updated address",
            ),
            audit_user_id=111,
            connected=True,
        )
        self.assertFalse(created)
        self.assertIsNotNone(officer2)
        self.assertEquals(officer2.username, username)
        self.assertEquals(officer2.last_name, "Last updated")
        self.assertEquals(officer2.other_names, "Other updated")
        self.assertEquals(officer2.audit_user_id, 111)
        self.assertTrue(officer2.has_login)
        self.assertFalse(officer2.phone_communication)
        self.assertEquals(officer2.location_id, 17)
        self.assertIsNone(officer2.substitution_officer_id)
        self.assertEquals(officer2.address, "updated address")
        self.assertEquals(str(officer2.works_to.date()), "2025-05-05")
        self.assertEquals(
            list(
                OfficerVillage.objects.filter(
                    validity_to__isnull=True, officer=officer2
                )
                .order_by("location_id")
                .values_list("location_id", flat=True)
            ),
            [22],
        )
        self.assertEquals(officer2.phone, "+00000")
        self.assertEquals(officer2.email, "imis@bar.be")

        deleted_officers = Officer.objects.filter(code=username).delete()
        logger.info(f"Deleted {deleted_officers} officers after test")

    def test_claim_admin_min(self):
        username = "tstsvca1"
        claim_admin, created = create_or_update_claim_admin(
            user_id=None,
            data=dict(
                username=username, last_name="Last Name CA1", other_names="Other 1 2 3"
            ),
            audit_user_id=999,
            connected=False,
        )
        self.assertTrue(created)
        self.assertIsNotNone(claim_admin)
        self.assertEquals(claim_admin.username, username)
        self.assertEquals(claim_admin.last_name, "Last Name CA1")
        self.assertEquals(claim_admin.other_names, "Other 1 2 3")

        deleted_officers = Officer.objects.filter(code=username).delete()
        logger.info(f"Deleted {deleted_officers} officers after test")

    def test_claim_admin_max(self):
        username = "tstsvca2"
        claim_admin, created = create_or_update_claim_admin(
            user_id=None,
            data=dict(
                username=username,
                last_name="Last Name CA2",
                other_names="Other 1 2 3",
                dob="1999-05-05",
                phone="+12345678",
                email="imis@foo.be",
                health_facility_id=1,
            ),
            audit_user_id=999,
            connected=True,
        )
        self.assertTrue(created)
        self.assertIsNotNone(claim_admin)
        self.assertEquals(claim_admin.username, username)
        self.assertEquals(claim_admin.last_name, "Last Name CA2")
        self.assertEquals(claim_admin.other_names, "Other 1 2 3")
        self.assertEquals(claim_admin.audit_user_id, 999)
        self.assertTrue(claim_admin.has_login)
        self.assertEquals(claim_admin.health_facility_id, 1)
        self.assertEquals(claim_admin.phone, "+12345678")
        self.assertEquals(claim_admin.email_id, "imis@foo.be")

        deleted_officers = Officer.objects.filter(code=username).delete()
        logger.info(f"Deleted {deleted_officers} officers after test")

    def test_user_reset_password(self):
        from django.core import mail

        roles = [11, 12]
        username = "user_reset"
        i_user, created = create_or_update_interactive_user(
            user_id=None,
            data=dict(
                username=username,
                last_name="LN",
                other_names="ON",
                roles=roles,
                language="fr",
                phone="+123456789",
                email=f"{username}@illuminati.int",
                health_facility_id=1,
                password=PASSWORD,
            ),
            audit_user_id=999,
            connected=False,
        )
        self.assertTrue(created)
        self.assertTrue(i_user.check_password(PASSWORD))

        # Core user necessary for the update
        core_user, core_user_created = create_or_update_core_user(
            None, username, i_user=i_user
        )
        request = self.factory.get("/")

        reset_user_password(request, username)

        self.assertTrue(len(mail.outbox) == 1)
        self.assertTrue(mail.outbox[0].subject == "[OpenIMIS] Reset Password")

        UserRole.objects.filter(user_id=i_user.id).delete()
        if connection.vendor == postgresql:
            UserRole.objects.filter(user_id=core_user.id).delete()
        core_user.delete()
        i_user.delete()

    def test_user_set_password(self):
        roles = [11, 12]
        username = "user_set"
        i_user, created = create_or_update_interactive_user(
            user_id=None,
            data=dict(
                username=username,
                last_name="LN",
                other_names="ON",
                roles=roles,
                language="fr",
                phone="+123456789",
                email=f"{username}@illuminati.int",
                health_facility_id=1,
                password=PASSWORD,
            ),
            audit_user_id=999,
            connected=False,
        )
        self.assertTrue(created)
        self.assertTrue(i_user.check_password(PASSWORD))
        # Core user necessary for the update
        core_user, _ = create_or_update_core_user(None, username, i_user=i_user)

        from django.core.exceptions import ValidationError

        request = self.factory.get("/")
        with self.assertRaises(ValidationError):
            set_user_password(request, username, "TOKEN", "new_password")

        UserRole.objects.filter(user_id=i_user.id).delete()
        if connection.vendor == postgresql:
            UserRole.objects.filter(user_id=core_user.id).delete()
        core_user.delete()
        i_user.delete()



from core.models.base import *
from core.models import versioned_model
from core.models import user
from core.models import history_model
from core.models import base_mutation
from core.models import user_mutation

_query_export_path = user._query_export_path
_get_default_expire_date = user._get_default_expire_date
User = user.User
UserRole = user.UserRole
VersionedModel = versioned_model.VersionedModel
BaseVersionedModel = versioned_model.BaseVersionedModel
HistoryModel = history_model.HistoryModel
HistoryBusinessModel = history_model.HistoryBusinessModel
HistoryModelManager = history_model.HistoryModelManager
MutationLog = base_mutation.MutationLog
UUIDVersionedModel = versioned_model.UUIDVersionedModel
InteractiveUser = user.InteractiveUser
TechnicalUser = user.TechnicalUser
Officer = user.Officer
ClaimAdmin = user.ClaimAdmin
Group = user.Group
RoleRight = user.RoleRight
Role = user.Role
ExportableQueryModel = base_mutation.ExportableQueryModel
UserMutation = user_mutation.UserMutation
RoleMutation = user_mutation.RoleMutation
ObjectMutation = base_mutation.ObjectMutation
CachedManager = versioned_model.CachedManager
